From 6f53d65f9ae5e1c61a2ca2650b149c895ce9794c Mon Sep 17 00:00:00 2001
From: Harm van der Vegt <harmvegt@gmail.com>
Date: Sun, 1 Sep 2024 20:03:34 +0200
Subject: [PATCH] Make chart-space-usage OS and shell independent

---
 lisp/emacs-lisp/chart.el | 41 +++++++++++++++++++++++++++++------------
 1 file changed, 29 insertions(+), 12 deletions(-)

diff --git a/lisp/emacs-lisp/chart.el b/lisp/emacs-lisp/chart.el
index c195ccb7165..b0302b55278 100644
--- a/lisp/emacs-lisp/chart.el
+++ b/lisp/emacs-lisp/chart.el
@@ -641,27 +641,44 @@ SORT-PRED if desired."
 		       (lambda (a b) (> (cdr a) (cdr b))))
     ))
 
+;; This assumes 4KB blocks
+(defun chart--file-size (size)
+  (* (/ (+ size 4095) 4096) 4096))
+
+(defun chart--directory-size (dir)
+  "Compute total size of files in directory DIR and its subdirectories.
+DIR is assumed to be a directory, verified by the caller."
+  (let ((total-size 0))
+    (dolist (file (directory-files dir t directory-files-no-dot-files-regexp))
+      (cond
+       ((file-regular-p file)
+        (setq total-size (+ total-size (chart--file-size (nth 7 (file-attributes file))))))
+       ((file-directory-p file)
+        (setq total-size (+ total-size (chart--directory-size file))))))
+    total-size))
+
 (defun chart-space-usage (d)
   "Display a top usage chart for directory D."
   (interactive "DDirectory: ")
   (message "Collecting statistics...")
   (let ((nmlst nil)
-        (cntlst nil))
-    (dolist (file (directory-files d t))
-      (when (file-regular-p file)
-        (let ((size (nth 7 (file-attributes file))))
+	(cntlst nil))
+
+    (dolist (file (directory-files d t directory-files-no-dot-files-regexp))
+        (let ((size (if (file-regular-p file)
+                        (nth 7 (file-attributes file))
+                      (chart--directory-size file))))
           (setq nmlst (cons (file-name-nondirectory file) nmlst))
-          (setq cntlst (cons size cntlst)))))
+          (setq cntlst (cons (chart--file-size size) cntlst))))
 
     (if (not nmlst)
-        (error "No files found!"))
-
-    ;; Display the chart if files are found
+	(error "No files found!"))
     (chart-bar-quickie 'vertical (format "Largest files in %s" d)
-                       nmlst "File Name"
-                       cntlst "File Size"
-                       10
-                       (lambda (a b) (> (cdr a) (cdr b))))))
+		       nmlst "File Name"
+		       cntlst "File Size"
+		       10
+		       (lambda (a b) (> (cdr a) (cdr b))))
+    ))
 
 (defun chart-emacs-storage ()
   "Chart the current storage requirements of Emacs."
-- 
2.11.0.windows.3

